# frozen_string_literal: true

# This file was auto-generated by lib/tasks/api.rake

module Usps
	module Api
		module Endpoints
			module IntlRateV2
				# The
				# IntlRateV2 API lets customers calculate the rate for international packages and
				# envelopes given the weight and dimensions of the item. The IntlRateV2 API
				# limits the data requested to twenty-five (25) packages per transaction. For
				# specifications such as package dimensions, delivery information, etc, please refer to the International Mail Manual (IMM) at
				# http://pe.usps.com/.
				# @param [Hash] options
				# @option options [required, Hash] intl_rate_v2_request API=IntlRateV2 This API returns the current international USPS postage corresponding to the parameters given.
				#  * *:revision* (String) — Set this value to “2” to return all currently documented response fields. For example: <Revision>2</Revsion>
				#  * *:package* (Hash) — Opening Package tag.
				#    * *:id* (required, NMTOKEN) — No restriction on number or type of characters provided valid XML syntax and unique to request. For example: <Package ID="0">...</Package>
				#    * *:pounds* (required, Integer) — Value must be numeric. Package weight generally cannot exceed 70 pounds. Maximum Decimal places are 8. Refer to the International Mail Manual (IMM) for weight requirements per country and mail service. The IMM can be found at the Postal Explorer web site. For example: <Pounds>2</Pounds> <Pounds>2.12345678</Pounds>
				#    * *:ounces* (required, Decimal) — Value must be numeric. Package weight generally cannot exceed 70 pounds. Maximum Decimal places are 8. Refer to the International Mail Manual (IMM) for weight requirements per country and mail service. The IMM can be found at the Postal Explorer web site. For example: <Ounces>4</Ounces> <Ounces>4.12345678</Ounces>
				#    * *:machinable* (Boolean) — Indicates whether or not the item is machinable. A surcharge is applied to a First-Class Mail International item if it has one or more non-machinable characteristics. See International Mail Manual (IMM) Section 241.217 for more information. For example: <Machinable>True</Machinable>
				#    * *:mail_type* (required, String) — Package type being shipped. For example: <MailType>Package</MailType>
				#    * *:gxg* (Hash) — If GXG rate is desired, then this group must be specified. Note that if this data precludes delivery, due to size or availability of service at the destination, then GXG rates simply will not be returned (not an error condition.)
				#      * *:po_box_flag* (required, String) — Specify as "Y" if the destination is a post office box. For example: <POBoxFlag>Y</POBoxFlag>
				#      * *:gift_flag* (required, String) — Specify as "Y" if the package contains a gift. For example: <GiftFlag>Y</GiftFlag>
				#      * *:value_of_contents* (required, String) — If specified, used to compute Insurance fee (if insurance is available for service and destination). For example: <ValueOfContents>103.00</ValueOfContents>
				#      * *:country* (required, String) — Entries must be from the USPS list of valid countries from the International Country Listings. To access the International Country Listings, go to the Index of Countries and Localities. For example: <Country>Albania</Country>
				#      * *:container* (String) — Note: This tag has been deprecated. <Container> tag and any values within the <Container> tag will not result in an error response and will not impact rates that are returned.
				#      * *:size* (String) — Note: This tag has been deprecated <Size> tag and any values within the <Size> tag will not result in an error response and will not impact rates that are returned.
				#      * *:width* (Integer) — Value must be numeric. Units are inches. If partial dimensions are provided, an error response will return. Length, Width, Height are required for accurate pricing of a rectangular package when any dimension of the item exceeds 12 inches. In addition, Girth is required only for a non-rectangular package in addition to Length, Width, Height when any dimension of the package exceeds 12 inches. For rectangular packages, the Girth dimension must be left blank as this dimension is to only be used for non-rectangular packages. For more details on dimensional weight pricing, please reference the Domestic Mail Manual Section 123.1.4 for Retail Mail and Section 223.1.6 for Commercial Mail. https://pe.usps.com/text/dmm300/index.htm
				#      * *:length* (Integer) — Value must be numeric. Units are inches. If partial dimensions are provided, an error response will return. Length, Width, Height are required for accurate pricing of a rectangular package when any dimension of the item exceeds 12 inches. In addition, Girth is required only for a non-rectangular package in addition to Length, Width, Height when any dimension of the package exceeds 12 inches. For rectangular packages, the Girth dimension must be left blank as this dimension is to only be used for non-rectangular packages. For more details on dimensional weight pricing, please reference the Domestic Mail Manual Section 123.1.4 for Retail Mail and Section 223.1.6 for Commercial Mail. https://pe.usps.com/text/dmm300/index.htm
				#      * *:height* (Integer) — Value must be numeric. Units are inches. If partial dimensions are provided, an error response will return. Length, Width, Height are required for accurate pricing of a rectangular package when any dimension of the item exceeds 12 inches. In addition, Girth is required only for a non-rectangular package in addition to Length, Width, Height when any dimension of the package exceeds 12 inches. For rectangular packages, the Girth dimension must be left blank as this dimension is to only be used for non-rectangular packages. For more details on dimensional weight pricing, please reference the Domestic Mail Manual Section 123.1.4 for Retail Mail and Section 223.1.6 for Commercial Mail. https://pe.usps.com/text/dmm300/index.htm
				#      * *:girth* (Integer) — Value must be numeric. Units are inches. If partial dimensions are provided, an error response will return. Length, Width, Height are required for accurate pricing of a rectangular package when any dimension of the item exceeds 12 inches. In addition, Girth is required only for a non-rectangular package in addition to Length, Width, Height when any dimension of the package exceeds 12 inches. For rectangular packages, the Girth dimension must be left blank as this dimension is to only be used for non-rectangular packages. For more details on dimensional weight pricing, please reference the Domestic Mail Manual Section 123.1.4 for Retail Mail and Section 223.1.6 for Commercial Mail. https://pe.usps.com/text/dmm300/index.htm
				#      * *:origin_zip* (String) — Available when IntlRateV2Request [Revision='2']. Origin ZIP Code is required to determine Priority Mail International price to Canadian destinations and is used to determine mail-ability of Global Express Guaranteed. When provided, the response will return a list of Post Office locations where GXG is accepted. The Origin ZIP Code must be valid. For example: <OriginZip>20770</OriginZip>
				#      * *:commercial_flag* (String) — Returns commercial base postage. For example: <CommercialFlag>Y<CommercialFlag>
				#      * *:commercial_plus_flag* (String) — Returns commercial plus postage. For example: <CommercialPlusFlag>Y<CommercialPlusFlag>
				#      * *:extra_services* (Hash) — Available when IntlRateV2Request[Revision='2']. Groups the ExtraService elements.
				#        * *:extra_service* (Extra Service Name) — [{"Extra Service Name"=>"Registered Mail", "ServiceID"=>"103"}, {"Extra Service Name"=>"Insurance – Global Express Guaranteed", "ServiceID"=>"106"}, {"Extra Service Name"=>"Insurance – Priority Mail International", "ServiceID"=>"108"}, {"Extra Service Name"=>"Return Receipt", "ServiceID"=>"105"}, {"Extra Service Name"=>"Certificate of Mailing", "ServiceID"=>"100"}, {"Extra Service Name"=>"Electronic USPS Delivery Confirmation International", "ServiceID"=>"109"}]
				#        * *:acceptance_date_time* (DateTime) — Available when IntlRateV2Request[Revision='2']. Date and Time the package is accepted by USPS. The AcceptanceDateTime tag along with the DestinationPostalCode and OriginZip is used to calculate the GuaranteeAvailability and also GuaranteeAvailability response tag for PMEI services in Kahala countries. ISO 8601 formatted date. YYYY-MM-DDThh:mm:ss+/-hh:mm For example, 2014-01-22T14:30:51-06:00
				#        * *:destination_postal_code* (String) — Available when IntlRateV2Request[Revision='2']. Destination Postal Code The AcceptanceDateTime tag along with the DestinationPostalCode and OriginZip is used to calculate the GuaranteeAvailability and also GuaranteeAvailability response tag for PMEI services in Kahala countries.
				#        * *:content* (Hash) — Used to describe the contents of the package.
				#          * *:content_type* (String) — Contains the enumerated description of the items in the package. “NonnegotiableDocument” and “Documents” both signify mailable non-negotiable documents and are insured automatically for up to $100, though Insurance will not be returned as an extra service. Additional Insurance cannot be purchased. Any non-document ContentType values are insured automatically for up to $200 and Insurance will be returned as an explicit extra service in the response. Additional Insurance can be purchased for values $200 and greater.
				#          * *:content_description* (String) — For future use
def intl_rate_v2(options = {})
					throw ArgumentError.new('Required arguments :intl_rate_v2_request missing') if options[:intl_rate_v2_request].nil?

					request = build_request(:intl_rate_v2, options)
					get('https://secure.shippingapis.com/ShippingAPI.dll', {
						API: 'IntlRateV2',
						XML: request,
					})
				end

				private

				def tag_unless_blank(xml, tag_name, data)
					xml.tag!(tag_name, data) unless data.blank? || data.nil?
				end

				def build_intl_rate_v2_request(xml, options = {})
					tag_unless_blank(xml, 'Revision', options[:intl_rate_v2_request][:revision])
					xml.tag!('Package') do
						xml.tag!('ID', options[:intl_rate_v2_request][:package][:id])
						xml.tag!('Pounds', options[:intl_rate_v2_request][:package][:pounds])
						xml.tag!('Ounces', options[:intl_rate_v2_request][:package][:ounces])
						tag_unless_blank(xml, 'Machinable', options[:intl_rate_v2_request][:package][:machinable])
						xml.tag!('MailType', options[:intl_rate_v2_request][:package][:mail_type])
						xml.tag!('GXG') do
							xml.tag!('POBoxFlag', options[:intl_rate_v2_request][:package][:gxg][:po_box_flag])
							xml.tag!('GiftFlag', options[:intl_rate_v2_request][:package][:gxg][:gift_flag])
							xml.tag!('ValueOfContents', options[:intl_rate_v2_request][:package][:gxg][:value_of_contents])
							xml.tag!('Country', options[:intl_rate_v2_request][:package][:gxg][:country])
							tag_unless_blank(xml, 'Container', options[:intl_rate_v2_request][:package][:gxg][:container])
							tag_unless_blank(xml, 'Size', options[:intl_rate_v2_request][:package][:gxg][:size])
							tag_unless_blank(xml, 'Width', options[:intl_rate_v2_request][:package][:gxg][:width])
							tag_unless_blank(xml, 'Length', options[:intl_rate_v2_request][:package][:gxg][:length])
							tag_unless_blank(xml, 'Height', options[:intl_rate_v2_request][:package][:gxg][:height])
							tag_unless_blank(xml, 'Girth', options[:intl_rate_v2_request][:package][:gxg][:girth])
							tag_unless_blank(xml, 'OriginZip', options[:intl_rate_v2_request][:package][:gxg][:origin_zip])
							tag_unless_blank(xml, 'CommercialFlag', options[:intl_rate_v2_request][:package][:gxg][:commercial_flag])
							tag_unless_blank(xml, 'CommercialPlusFlag', options[:intl_rate_v2_request][:package][:gxg][:commercial_plus_flag])
							xml.tag!('ExtraServices') do
								tag_unless_blank(xml, 'ExtraService', options[:intl_rate_v2_request][:package][:gxg][:extra_services][:extra_service])
								tag_unless_blank(xml, 'AcceptanceDateTime', options[:intl_rate_v2_request][:package][:gxg][:extra_services][:acceptance_date_time])
								tag_unless_blank(xml, 'DestinationPostalCode', options[:intl_rate_v2_request][:package][:gxg][:extra_services][:destination_postal_code])
								xml.tag!('Content') do
									tag_unless_blank(xml, 'ContentType', options[:intl_rate_v2_request][:package][:gxg][:extra_services][:content][:content_type])
									tag_unless_blank(xml, 'ContentDescription', options[:intl_rate_v2_request][:package][:gxg][:extra_services][:content][:content_description])
								end if options[:intl_rate_v2_request][:package][:gxg][:extra_services][:content].present?
							end if options[:intl_rate_v2_request][:package][:gxg][:extra_services].present?
						end if options[:intl_rate_v2_request][:package][:gxg].present?
					end if options[:intl_rate_v2_request][:package].present?
					xml.target!
				end

			end
		end
	end
end
